
/* Marcelo Queiroz, junho/2011        */
/* baseado em d_misc.c do fonte do Pd */

#include "m_pd.h"
#include <stdio.h>
#include <string.h>

/* ------------------------- fprintf~ -------------------------- */
static t_class *fprintf_class;

typedef struct _fprintf
{
    t_object x_obj;
    t_float x_f;
    t_symbol *x_sym;
    int x_count;
    int x_stat;
    int x_first;
    int x_last;
    FILE *x_hndl;
} t_fprintf;

static t_int *fprintf_perform(t_int *w)
{
    t_fprintf *x = (t_fprintf *)(w[1]);
    t_sample *in = (t_sample *)(w[2]);
    t_sample *in2 = (t_sample *)(w[3]);
    int n = (int)(w[4]);
    int last = (x->x_last==-1)?n-1:x->x_last;
     
    if (x->x_stat)
    {
        int i;
        for(i=x->x_first; i<=last; i++) {
	  fprintf(x->x_hndl,"%-8.5g ",in[i]);
        }
	fprintf(x->x_hndl,"\n");
	for(i=x->x_first; i<=last; i++) {
	  fprintf(x->x_hndl,"%-8.5g ",in2[i]);
	}
	fprintf(x->x_hndl,"\n");
        if (x->x_count)
	  {
	    x->x_count--;
	    if (!x->x_count) x->x_stat = 0;
	  }
    }
    return (w+5);
}

static void fprintf_dsp(t_fprintf *x, t_signal **sp)
{
  dsp_add(fprintf_perform, 4, x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}

static void fprintf_float(t_fprintf *x, t_float f)
{
  if (x->x_hndl)
    {
      fprintf(x->x_hndl,"%d\n",(int)f);
    }
}

static void fprintf_bang(t_fprintf *x)
{
  if (x->x_hndl)
    x->x_count = 1;
}

static void fprintf_anything(t_fprintf *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!strcmp(s->s_name,"open"))
    {
      t_symbol *n = atom_getsymbol(&argv[0]);
      x->x_hndl = fopen(n->s_name,"w");
      if (!x->x_hndl)
	post("fprintf~ error: Cannot open file");
    }
  else if (!strcmp(s->s_name,"start"))
    {
      if (x->x_hndl)
	x->x_stat = 1;
      else post("fprintf~ error: please open a file first");
    }
  else if (!strcmp(s->s_name,"stop"))
    {
      if (x->x_hndl)
	x->x_stat = 0;
      else post("fprintf~ error: please open a file first");
    }
  else if (!strcmp(s->s_name,"close"))
    {
      if (x->x_hndl)
	{ 
	  x->x_stat = 0;
	  fflush(x->x_hndl);
	  fclose(x->x_hndl);
	  x->x_hndl = NULL;
	}
      else post("fprintf~ error: please open a file first");
    }
  else if (!strcmp(s->s_name,"first"))
    {
      x->x_first = atom_getint(&argv[0]);
    }
  else if (!strcmp(s->s_name,"last"))
    {
      x->x_last = atom_getint(&argv[0]);
    }
  else if (!strcmp(s->s_name,"write"))
    fprintf_float(x,atom_getint(&argv[0]));
  else if (!strcmp(s->s_name,"repeat"))
    {
      int f = atom_getint(&argv[0]);
      if (f<0) f=0;
      x->x_count = f;
      x->x_stat = 1;
    }
  else {
    static char mess[200];
    sprintf(mess,"fprintf~ error: I do not understand %s",s->s_name);
    post(mess);
  }
}

static void *fprintf_new(t_symbol *s)
{
    t_fprintf *x = (t_fprintf *)pd_new(fprintf_class);
    x->x_sym = (s->s_name[0]? s : gensym("fprintf~"));
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
    x->x_count = 0;
    x->x_stat = 0;
    x->x_first = 0;
    x->x_last = -1; /* this means "end of the dsp block" */
    x->x_f = 0;
    x->x_hndl = NULL;
    return (x);
}

void fprintf_tilde_setup(void)
{
    fprintf_class = class_new(gensym("fprintf~"), (t_newmethod)fprintf_new, 0,
        sizeof(t_fprintf), 0, A_DEFSYM, 0);
    CLASS_MAINSIGNALIN(fprintf_class, t_fprintf, x_f);
    class_addmethod(fprintf_class, (t_method)fprintf_dsp, gensym("dsp"), 0);
    class_addbang(fprintf_class, fprintf_bang);
    class_addfloat(fprintf_class, fprintf_float);
    class_addanything(fprintf_class, fprintf_anything);
 
}





